from .questions import QUESTIONS, QUESTIONS_KR
from .info import OEJTS_INFO, OEJTS_INFO_KR

#dictionary macros.
QUESTIONS           = dict(sorted(QUESTIONS.items()))
QUESTIONS_KR        = dict(sorted(QUESTIONS_KR.items()))
Q_QUANTITY          = len(QUESTIONS)
Q_INDEXES           = list(QUESTIONS.keys())

#gui setting constants.
ROOT_WIN_MINSIZE    = (497, 270)
ABOUT_WIN_MINSIZE   = (400, 500)
RESULT_WIN_MINSIZE  = (300, 169)

#symbolic constants.
EN                  = 'en'
KR                  = 'kr'
IE                  = 'IE'
SN                  = 'SN'
FT                  = 'FT'
JP                  = 'JP'
MBTI                = 'MBTI'
LEFT                = 0
RIGHT               = 1
P_MEDIAN            = 3
UNDERLINE_EN        = 0
UNDERLINE_DEFAULT   = -1
ABOUT               = 0x00
PREV                = 0x01
NEXT                = 0x02
ABOUT_TITLE         = 0x03
RESULT_TITLE        = 0x04
OKAY                = 0x05
INFO                = 0x06
HEADER              = 0x07
RESULT              = 0x08



#user interface texts.
UI_TXT_EN = {
    ABOUT           : 'About',
    PREV            : 'Previous',
    NEXT            : 'Next',
    ABOUT_TITLE     : 'About',
    RESULT_TITLE    : 'Result',
    OKAY            : 'Ok',
    INFO            : OEJTS_INFO,
    HEADER          : (lambda i : f"Question {i}"),
    RESULT          : (lambda ie, sn, ft, jp, mbti : f"\n[Result]\n\nIE: {ie}  SN: {sn}  FT: {ft}  JP: {jp}\n\nYou're {mbti}!") 
}

UI_TXT_KR = {
    ABOUT           : '도움말(A)',
    PREV            : '이전(P)',
    NEXT            : '다음(N)',
    ABOUT_TITLE     : '도움말',
    RESULT_TITLE    : '결과',
    OKAY            : '확인(O)',
    INFO            : OEJTS_INFO_KR,
    HEADER          : (lambda i : f"{i}번째 질문"),
    RESULT          : (lambda ie, sn, ft, jp, mbti : f"\n[결과]\n\nIE: {ie}  SN: {sn}  FT: {ft}  JP: {jp}\n\n당신은 {mbti}이네요!")
}


QUESTIONS = {
    "Q01" : ['''Makes lists''', '''Relies on memory'''],
    "Q02" : ['''Sceptical''', '''Wants to believe'''],
    "Q03" : ['''Bored by time alone''', '''Needs time alone'''],
    "Q04" : ['''Accepts things as they are''', '''Unsatisfied with the ways things are'''],
    "Q05" : ['''Keeps a clean room''', '''Just puts stuff where ever'''],
    "Q06" : ['''Thinks "robotic" is an insult''', '''Strives to have a mechanical mind'''],
    "Q07" : ['''Energetic''', '''Mellow'''],
    "Q08" : ['''Prefer to take multiple choice test''', '''Prefer essay answers'''],
    "Q09" : ['''Chaotic''', '''Organized'''],
    "Q10" : ['''Easily hurt''', '''Thick-skinned'''],
    "Q11" : ['''Works best in groups''', '''Works best alone'''],
    "Q12" : ['''Focused on the present''', '''Focused on the future'''],
    "Q13" : ['''Plans far ahead''', '''Plans at the last minute'''],
    "Q14" : ['''Wants people's respect''', '''Wants their love'''],
    "Q15" : ['''Gets worn out by parties''', '''Gets fired up by parties'''],
    "Q16" : ['''Fits in''', '''Stands out'''],
    "Q17" : ['''Keeps options open''', '''Commits'''],
    "Q18" : ['''Wants to be good at fixing things''', '''Wants to be good at fixing people'''],
    "Q19" : ['''Talks more''', '''Listens more'''],
    "Q20" : ['''When describing an event, will tell people what happened''', '''When describing an event, will tell people what it meant'''],
    "Q21" : ['''Gets work done right away''', '''Procrastinates'''],
    "Q22" : ['''Follows the heart''', '''Follows the head'''],
    "Q23" : ['''Stays at home''', '''Goes out on the town'''],
    "Q24" : ['''Wants the big picture''', '''Wants the details'''],
    "Q25" : ['''Improvises''', '''Prepares'''],
    "Q26" : ['''Bases morality on justice''', '''Bases morality on compassion'''],
    "Q27" : ['''Finds it difficult to yell very loudly''', '''Yelling to others when they are faraway comes naturally'''],
    "Q28" : ['''Theoretical''', '''Empirical'''],
    "Q29" : ['''Works hard''', '''Plays hard'''],
    "Q30" : ['''Uncomfortable with emotions''', '''Values emotions'''],
    "Q31" : ['''Likes to perform in front of other people''', '''Avoids public speaking'''],
    "Q32" : ['''Likes to know "Who?", "What?", "When?"''', '''Likes to know "Why?"''']
}



QUESTIONS_KR = {
    "Q01": ["목록을 만든다", "기억에 의존한다"],
    "Q02": ["회의적이다", "믿고 싶어 한다"],
    "Q03": ["혼자 있으면 지루하다", "혼자 있는 시간이 필요하다"],
    "Q04": ["있는 그대로 받아들인다", "현실에 만족하지 않는다"],
    "Q05": ["방을 깨끗하게 유지한다", "물건을 아무 데나 둔다"],
    "Q06": ["'기계적이다'는 말이 모욕처럼 느껴진다", "기계적인 사고를 추구한다"],
    "Q07": ["에너지가 넘친다", "차분하다"],
    "Q08": ["객관식 시험을 선호한다", "서술형을 선호한다"],
    "Q09": ["혼란스럽다", "정돈되어 있다"],
    "Q10": ["상처를 쉽게 받는다", "둔감한 편이다"],
    "Q11": ["여럿과 함께 일할 때 가장 잘한다", "혼자 일할 때 가장 잘한다"],
    "Q12": ["현재에 집중한다", "미래에 집중한다"],
    "Q13": ["먼저 계획을 세워둔다", "막판에 계획한다"],
    "Q14": ["사람들의 존중을 원한다", "사람들의 사랑을 원한다"],
    "Q15": ["파티에 가면 쉽게 지친다", "파티가 에너지를 준다"],
    "Q16": ["주변에 잘 섞인다", "두드러지는 편이다"],
    "Q17": ["옵션을 열어 둔다", "결단하고 확정하는 편이다"],
    "Q18": ["물건을 잘 고치는 사람이 되고 싶다", "사람을 잘 돌보는 사람이 되고 싶다"],
    "Q19": ["말을 더 많이 한다", "듣는 편이 더 많다"],
    "Q20": ["일어난 일을 중심으로 설명한다", "그 일이 의미하는 바를 중심으로 설명한다"],
    "Q21": ["일을 바로 처리한다", "미루는 편이다"],
    "Q22": ["마음을 따른다", "머리(이성)를 따른다"],
    "Q23": ["집에 머무는 것을 좋아한다", "외출해 돌아다니는 것을 좋아한다"],
    "Q24": ["큰 그림을 알고 싶어 한다", "세부 사항을 알고 싶어 한다"],
    "Q25": ["즉흥적으로 한다", "미리 준비한다"],
    "Q26": ["정의를 기준으로 도덕을 판단한다", "연민을 기준으로 도덕을 판단한다"],
    "Q27": ["크게 소리치기 어렵다", "멀리 있는 사람에게도 자연스럽게 소리칠 수 있다"],
    "Q28": ["이론적이다", "경험적이다"],
    "Q29": ["열심히 일한다", "열심히 논다"],
    "Q30": ["감정을 다루는 것이 불편하다", "감정을 중요하게 여긴다"],
    "Q31": ["사람들 앞에서 공연하기를 좋아한다", "사람들 앞에서 말하는 것을 피한다"],
    "Q32": ["'누가', '무엇을', '언제'가 궁금하다", "'왜'가 궁금하다"]
}







from .src import *

class IoejtsCalculator():
    
    def calc_p_s(self, raw_score):
        """
        Parameter:
            raw_score   : list
            list of Q_QUANTITY(normally 32.) elements;
            raw score of what user answered for the questions. 
        --------
        Return:
            : dict
            dictionary of 4 elements; calculated score of
            numeric representation of personality traits.
        """
        t = {
            IE : 0,
            SN : 0,
            FT : 0,
            JP : 0
        }

        distributor = lambda k : [raw_score[i] for i in range(k-1, Q_QUANTITY, 4)]
        
        a, b, c, d, e, f, g, h = distributor(3)
        t[IE] = 30-a-b-c+d-e+f+g-h

        a, b, c, d, e, f, g, h = distributor(4)
        t[SN] = 12+a+b+c+d+e-f-g+h

        a, b, c, d, e, f, g, h = distributor(2)
        t[FT] = 30-a+b+c-d-e+g-g-h

        a, b, c, d, e, f, g, h = distributor(1)
        t[JP] = 18+a+b-c+d-e+f-g+h

        return t

    def calc_mbti(self, score):
        """
        Parameter:
            score   : dict
            dictionary of scores of each personality traits.
        --------
        Return:
            : str
            result of the test in Myers-Briggs Type Indicator
        """
        c = []

        c.append('E' if score[IE] > 24 else 'I')
        c.append('N' if score[SN] > 24 else 'S')
        c.append('T' if score[FT] > 24 else 'F')
        c.append('P' if score[JP] > 24 else 'J')

        return "".join(c)




""" ioejts - Instantly implementable OEJTS

OEJTS v1.2 Personality test suite & API of quick and easy implementation.
All test and scoring methods are directly extracted from 
Jorgenson, E. (2015). Open Extended Jungian Type Scales v1.2.
Please refer to https://openpsychometrics.org/tests/OJTS/development/ for details.

License for OEJTS:
The items of the Open Extended Jungian Type Scales 1.2 are licenced under a Creative Commons
Attribution-NonCommercial-ShareAlike 4.0 International License. The OEJTS come with no
guarantees of reliability or accuracy of any kind

Usage:
>>>from ioejts import ioejts
>>>quick_test = ioejts()
    ...
>>>score_calc = ioejts(score: iter.)
    ...
>>>question_dict = ioejts(None, False, ...)
    ...
>>>ioejts()['MBTI']
ISFJ
    ...
"""
from collections import UserDict

from .util import IoejtsCalculator
from .gui import IoejtsSurveyWindow
from .src import *


class ioejts(UserDict, IoejtsCalculator):
    """instance of ioejts act as dictionary as result of wrapped by collections.UserDict,
    all default python dictionary behavior applied in the same manner.
    you can get few different contents as a dictionary through this class.

    Parameter:
        score       : list
            list of completed answer for the 32 OEJTS questions.
            if this parameter is given, then survey(or test) suit will
            automatically not be conducted and calculated result of
            given answer will be returned.
            Default is None.
        survey      : bool
            boolean value for deciding whether survey(or test) suit
            would be conducted or not. If true, additional graphical window
            will pop-up for the survey.
            Default is True.
        lang        : Symbolic Constants. (str)
            designator for what language the instance should provide.
            both survey suite(if conducted) and result will be provided as
            designated language.
            Default is EN.
        show_result : bool
            boolean value for deciding whether result page at the end
            of the survey suit will be displayed or not.
            Default is True.
    ----------------
    Returns:
            : dict
            dictionary of calculated results and regarding MBTI string, if 
            score has given or survey suit finished, or dictionary of questions
            if both is not given/False.
    """

    def __init__(self, score=None, survey=True, lang=EN, show_result=True):

        raw_score = []

        if score:
            raw_score = score
        elif survey:
            _wSurvey = IoejtsSurveyWindow(lang=lang, result_page=show_result)
            _wSurvey.mainloop()
            raw_score = _wSurvey.getResult()
        else:
            q = QUESTIONS_KR if lang == KR else QUESTIONS
            self._d = {**q}

        if raw_score:
            s = super().calc_p_s(raw_score)
            self._d = {**s, MBTI : super().calc_mbti(s)}

        super().__init__(self._d)


OEJTS_INFO = """Open Extended Jungian Type Scales 1.2


Introduction

This is a personality test that will give you a result equivalent to the Myers-Briggs Type Indicator, although this test is not the MBTI and has no affiliation with it.


Instructions

On each page, there are 32 pairs of personality descriptions connected by a five point scale. For each pair, you must choose where on the scale between them you think you are. For example, if the pair is “angry” versus “calm”, you should choose far left on the scale if you are always angry and never calm, a middle if you are half and half, etc.


Licence

The items of the Open Extended Jungian Type Scales 1.2 are licenced under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License. The OEJTS come with no guarantees of reliability or accuracy of any kind.


For more information, visit https://openpsychometrics.org/tests/OJTS/development/"""





OEJTS_INFO_KR = """Open Extended Jungian Type Scales 1.2


소개

이 검사는 마이어스-브릭스 유형 지표(MBTI)와 유사한 결과를 제공하는 성격 검사입니다. 다만 이 검사는 MBTI 자체가 아니며, MBTI와 어떠한 공식적인 관련도 없습니다.

검사 방법

각 페이지에는 다섯 단계 척도로 연결된 32개의 성격 묘사 쌍이 제시됩니다. 각 쌍을 읽고, 두 특성 사이에서 자신이 가장 가깝다고 생각되는 위치를 선택하면 됩니다.
예를 들어, 제시된 쌍이 “화를 잘 내는 편”과 “차분한 편”일 때, 늘 화를 내고 거의 차분하지 않으면 척도의 가장 왼쪽을 선택하고, 두 성향이 비슷하다면 가운데를 선택하는 식입니다.

라이선스

Open Extended Jungian Type Scales 1.2의 문항은 Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License의 적용을 받습니다.
OEJTS는 신뢰도나 정확성에 대해 어떤 보증도 제공하지 않습니다.

자세한 정보는 https://openpsychometrics.org/tests/OJTS/development/ 를 방문하십시오."""




import tkinter as tk
import tkinter.ttk as ttk
from tkinter import scrolledtext


from .util import IoejtsCalculator
from .src import *


class IoejtsSurveyWindow(tk.Frame, IoejtsCalculator):

    def __init__(self, master=None, lang=EN, result_page=True):
        """ VARIABLE INITIALIZATION """
        self.result = []
        self._rp = result_page
        self.scaleValueList = [P_MEDIAN] * Q_QUANTITY
        self.isAboutTopLevelExist = False
        self.isResultTopLevelExist = False

        """ LOCALE SET """
        if lang == KR:
            self.QUESTIONS = QUESTIONS_KR
            self.UI_TXT = UI_TXT_KR
            self.UNDERLINE = UNDERLINE_DEFAULT
        else:
            self.QUESTIONS = QUESTIONS
            self.UI_TXT = UI_TXT_EN
            self.UNDERLINE = UNDERLINE_EN
        
        """ TOP-LEVEL FRAME CREATION """
        tk.Frame.__init__(self, master)
        self.grid(sticky=tk.N+tk.S+tk.E+tk.W)
        
        """ SURVEY APPLICATION CREATION """
        self.RootCreation()
        self.SurveyWidgetCreation()


    # interfaces.
    def getResult(self):
        return self.result


    #
    # application creation functions.
    #
    def RootCreation(self):
        self.root = self.winfo_toplevel()
        self.root.title('OEJTS')
        self.root.minsize(*ROOT_WIN_MINSIZE)
        self.root.lift()
        self.root.rowconfigure(0, weight=1)
        self.root.columnconfigure(0, weight=1)

        """ Keyboard Shortcut Bindings """
        self.root.bind("<KeyPress-a>", lambda e : self._AboutButtonUpdate(e))
        self.root.bind("<KeyPress-p>", lambda e : self._PreviousButtonUpdate(e))
        self.root.bind("<KeyPress-n>", lambda e : self._NextButtonUpdate(e))
        self.root.bind("<Right>", lambda e : self.widgetUserScale.set(self.widgetUserScale.get() + 1))
        self.root.bind("<Left>", lambda e : self.widgetUserScale.set(self.widgetUserScale.get() - 1))

    def SurveyWidgetCreation(self):
        """ Initializations """
        self.__SaveResult()


        """ Widget Control Variables """
        self.indexNumberInt = tk.IntVar()
        self.progressInt = tk.IntVar()
        self.userScaleInt = tk.IntVar()
        self.questionIndexString = tk.StringVar()
        self.leftQuestionString = tk.StringVar()
        self.rightQuestionString = tk.StringVar()


        """ Control Variable Initializations """
        self.indexNumberInt.set(0)
        self.progressInt.set(0)
        self.userScaleInt.set(3)
        self.questionIndexString.set(self.UI_TXT[HEADER](self.indexNumberInt.get() + 1))
        self.leftQuestionString.set(self.QUESTIONS[Q_INDEXES[0]][LEFT])
        self.rightQuestionString.set(self.QUESTIONS[Q_INDEXES[0]][RIGHT])


        """ Application Widget Grid Configurations """
        self.rowconfigure(0, minsize=30)
        self.rowconfigure(2, minsize=15)
        self.rowconfigure(4, minsize=15)
        self.rowconfigure(6, minsize=15)
        self.rowconfigure(7, weight=1, minsize=40)
        self.rowconfigure(8, minsize=15)
        self.rowconfigure(10, minsize=15)
        self.rowconfigure(12, minsize=30)

        self.columnconfigure(0, minsize=30)
        self.columnconfigure(2, minsize=30)
        self.columnconfigure(3, weight=1, minsize=100)
        self.columnconfigure(5, weight=1, minsize=100)
        self.columnconfigure(6, minsize=30)
        self.columnconfigure(8, minsize=30)


        """ Static Widgets """
        self.widgetTopSeparator = ttk.Separator(self, orient=tk.HORIZONTAL)
        self.widgetMiddleSeparator = ttk.Separator(self, orient=tk.VERTICAL)
        self.widgetBottomSeparator = ttk.Separator(self, orient=tk.HORIZONTAL)

        self.widgetTopSeparator.grid(row=5, column=1, columnspan=7, padx=5, sticky=tk.E+tk.W)
        self.widgetMiddleSeparator.grid(row=7, column=4, pady=5, sticky=tk.N+tk.S)
        self.widgetBottomSeparator.grid(row=9, column=1, columnspan=7, padx=5, sticky=tk.E+tk.W)


        """ Text Outputs """
        self.widgetQuestionIndex = tk.Label(self, font=('MS Serif', 14), 
                                                  textvariable=self.questionIndexString)
        self.widgetLeftQuestion = tk.Text(self, font=('Helvetica', 10), 
                                                wrap=tk.WORD, 
                                                state=tk.DISABLED, 
                                                width=15, 
                                                height=5, 
                                                relief=tk.FLAT, 
                                                bg=self.root.cget('bg'), 
                                                bd=0)
        self.widgetRightQuestion = tk.Text(self, font=('Helvetica', 10), 
                                                 wrap=tk.WORD, 
                                                 state=tk.DISABLED, 
                                                 width=15, 
                                                 height=5, 
                                                 relief=tk.FLAT, 
                                                 bg=self.root.cget('bg'), 
                                                 bd=0)
        
        self.widgetQuestionIndex.grid(row=0, rowspan=3, column=3, columnspan=3)
        self.widgetLeftQuestion.grid(row=7, column=2, columnspan=2, padx=15, sticky=tk.E+tk.W)
        self.widgetRightQuestion.grid(row=7, column=5, columnspan=2, padx=15, sticky=tk.E+tk.W)

        self.__TkStaticTextRewriter(self.widgetLeftQuestion, self.leftQuestionString.get())
        self.__TkStaticTextRewriter(self.widgetRightQuestion, self.rightQuestionString.get())
        
        
        """ Buttons """
        self.widgetAboutButton = ttk.Button(self, text=self.UI_TXT[ABOUT], 
                                                 underline=self.UNDERLINE, 
                                                 command=self._AboutButtonUpdate)
        self.widgetPreviousButton = ttk.Button(self, text=self.UI_TXT[PREV], 
                                                     underline=self.UNDERLINE, 
                                                     command=self._PreviousButtonUpdate)
        self.widgetNextButton = ttk.Button(self, text=self.UI_TXT[NEXT], 
                                                 underline=self.UNDERLINE, 
                                                 command=self._NextButtonUpdate)

        self.widgetAboutButton.grid(row=0, column=0, columnspan=2, padx=10, pady=10)
        self.widgetPreviousButton.grid(row=11, column=1)
        self.widgetNextButton.grid(row=11, column=7)


        """ Other Dynamic Widgets """
        self.widgetProgressbar = ttk.Progressbar(self, maximum=32, 
                                                       mode='determinate', 
                                                       orient=tk.HORIZONTAL, 
                                                       variable=self.progressInt)
        self.widgetUserScale = ttk.Scale(self, orient=tk.HORIZONTAL, 
                                               length=200, 
                                               from_=(P_MEDIAN-2), 
                                               to=(P_MEDIAN+2),
                                               variable=self.userScaleInt, 
                                               command=self._UserScaleUpdate)

        self.widgetProgressbar.grid(row=3, column=3, columnspan=3, sticky=tk.E+tk.W)
        self.widgetUserScale.grid(row=11, column=3, columnspan=3)


    #
    # widget command functions for application logics.
    #
    def _AboutButtonUpdate(self, e=None):
        if not self.isAboutTopLevelExist:
            self.isAboutTopLevelExist = True
            self.__AboutTopLevel()
            
    def _PreviousButtonUpdate(self, e=None):
        i = self.indexNumberInt.get()

        if i == 0:
            pass
        elif i in range(1, len(Q_INDEXES) + 1):
            self.__TkStaticTextRewriter(self.widgetLeftQuestion, self.QUESTIONS[Q_INDEXES[i - 1]][LEFT])
            self.__TkStaticTextRewriter(self.widgetRightQuestion, self.QUESTIONS[Q_INDEXES[i - 1]][RIGHT])

            self.indexNumberInt.set(i - 1)
            self.questionIndexString.set(self.UI_TXT[HEADER](self.indexNumberInt.get() + 1))

            self.userScaleInt.set(self.scaleValueList[i - 1])
        else:
            pass

    def _NextButtonUpdate(self, e=None):
        i = self.indexNumberInt.get()

        if i >= len(Q_INDEXES) - 1:
            self.__SaveResult()
            if self._rp:
                if not self.isResultTopLevelExist:
                    self.isResultTopLevelExist = True
                    self.__ResultTopLevel()
            else:
                self.__RootWindowTermination()
        elif i in range(len(Q_INDEXES) - 1):
            self.__TkStaticTextRewriter(self.widgetLeftQuestion, self.QUESTIONS[Q_INDEXES[i + 1]][LEFT])
            self.__TkStaticTextRewriter(self.widgetRightQuestion, self.QUESTIONS[Q_INDEXES[i + 1]][RIGHT])

            if i == self.progressInt.get():
                self.progressInt.set(self.progressInt.get() + 1)

            self.indexNumberInt.set(i + 1)
            self.questionIndexString.set(self.UI_TXT[HEADER](self.indexNumberInt.get() + 1))

            self.userScaleInt.set(self.scaleValueList[i + 1])
        else:
            pass

    def _UserScaleUpdate(self, e=None):
        self.__TtkScaleResolutionModifier(self.widgetUserScale)
        self.scaleValueList[self.indexNumberInt.get()] = self.userScaleInt.get()


    #
    # utility functions.
    #
    def __TkStaticTextRewriter(self, text, string):
        text.configure(state=tk.NORMAL)
        text.delete(0.0, tk.END)
        text.tag_configure("center", justify='center')
        text.insert("1.0", string)
        text.tag_add("center", "1.0", "end")
        text.configure(state=tk.DISABLED)

    def __TtkScaleResolutionModifier(self, scale):
        value = scale.get()
        if int(value) != value:
            scale.set(round(value))

    def __SaveResult(self):
        self.result = self.scaleValueList[::]

    def __RootWindowTermination(self):
        self.root.destroy()

    def __AboutTopLevel(self):
        """ Top-level Configuration """
        AboutTopLevel = tk.Toplevel()
        AboutTopLevel.title(self.UI_TXT[ABOUT_TITLE])
        AboutTopLevel.minsize(*ABOUT_WIN_MINSIZE)
        AboutTopLevel.transient(self)
        AboutTopLevel.focus_set()

        AboutTopLevel.rowconfigure(0, weight=1)
        AboutTopLevel.columnconfigure(0, weight=1)

        """ Textbox """
        infoTextBox = scrolledtext.ScrolledText(AboutTopLevel, font=('Helvetica', 10), 
                                                               wrap=tk.WORD, 
                                                               width=15, 
                                                               height=5,
                                                               state=tk.DISABLED,
                                                               relief=tk.SUNKEN, 
                                                               bg=AboutTopLevel.cget('bg'))
        infoTextBox.grid(row=0, column=0, sticky=tk.E+tk.W+tk.N+tk.S)
        self.__TkStaticTextRewriter(infoTextBox, self.UI_TXT[INFO])

        """ Ok Button """
        def closeTopLevel():
            AboutTopLevel.destroy()
            self.isAboutTopLevelExist = False

        AboutTopLevel.protocol("WM_DELETE_WINDOW", closeTopLevel)
        okayButton = ttk.Button(AboutTopLevel, text=self.UI_TXT[OKAY], 
                                                    underline=self.UNDERLINE, 
                                                    command=closeTopLevel)
        okayButton.grid(row=1, column=0, padx=15, pady= 15, sticky=tk.E)

        """ Keyboard Shortcut Bindings """        
        AboutTopLevel.bind("<KeyPress-o>", lambda e : closeTopLevel())
        AboutTopLevel.bind("<Escape>", lambda e : closeTopLevel())

    def __ResultTopLevel(self):
        """ Top-level Configuration """
        ResultTopLevel = tk.Toplevel()
        ResultTopLevel.title(self.UI_TXT[RESULT_TITLE])
        ResultTopLevel.minsize(*RESULT_WIN_MINSIZE)
        ResultTopLevel.transient(self)
        ResultTopLevel.grab_set()
        ResultTopLevel.focus_set()

        ResultTopLevel.rowconfigure(0, weight=1)
        ResultTopLevel.columnconfigure(0, weight=1)

        """ Textbox """
        infoTextBox = tk.Text(ResultTopLevel, font=('Helvetica', 10), 
                                              wrap=tk.WORD, 
                                              width=20, 
                                              height=7,
                                              state=tk.DISABLED,
                                              relief=tk.FLAT, 
                                              bg=ResultTopLevel.cget('bg'),
                                              bd=0)
        infoTextBox.grid(row=0, column=0, sticky=tk.E+tk.W)

        s = super().calc_p_s(self.getResult())
        self.__TkStaticTextRewriter(infoTextBox, self.UI_TXT[RESULT](*[*s.values()], "".join(super().calc_mbti(s))))

        """ Ok Button """
        def closeTopLevel():
            self.__RootWindowTermination()

        ResultTopLevel.protocol("WM_DELETE_WINDOW", closeTopLevel)
        okayButton = ttk.Button(ResultTopLevel, text=self.UI_TXT[OKAY], 
                                                     underline=self.UNDERLINE, 
                                                     command=closeTopLevel)
        okayButton.grid(row=1, column=0, padx=15, pady= 15, sticky=tk.E)

        """ Keyboard Shortcut Bindings """
        ResultTopLevel.bind("<KeyPress-o>", lambda e : closeTopLevel())
        ResultTopLevel.bind("<Escape>", lambda e : closeTopLevel())

